#![allow(dead_code)]

mod device;
mod vm;

use device::{Reader, Arithmetic, Tape, Writer};
use std::io::Cursor;
use rand::prelude::random;
use std::io::Read;
use std::io;
use vm::{State, vm_run};


fn main() {

     let program: Vec<u8> = io::stdin().bytes().map(|res| res.unwrap_or_else(|_| std::process::exit(1))).collect();


     for _ in 0..10 {
         let iv: [u8; 4] = random();
         let mut input = Cursor::new(iv);
         let mut output = Cursor::new(vec![0,0,0,0]);

         let state = State {
             ip: 0usize,
             mem: [0; 4],
             program: program.clone(),
             devices: vec![Box::new(Reader { reader: &mut input } ), 
                           Box::new(Writer { writer: &mut output } ), 
                           Box::new(Arithmetic { buffer: [0; 3], index: 0usize }),
                           Box::new(Tape { tape: [0; 25000], index: 0usize }),]
         };
         vm_run(state);
         let ov: [u8; 4] = output.into_inner().try_into().unwrap();
         println!("{:?}", iv);
         println!("{:?}", encrypt(iv));
         println!("{:?}", ov);
         if ov != encrypt(iv) {
             std::process::exit(1);
         }
     }

}


fn encrypt(block: [u8; 4]) -> [u8; 4] {
    let mut block = block;

    let (i, _) = block[0].overflowing_add(block[3]);
    block[1] ^= i;
    let (i, _) = block[1].overflowing_add(block[0]);
    block[2] ^= i;
    let (i, _) = block[2].overflowing_add(block[1]);
    block[3] ^= i;
    let (i, _) = block[3].overflowing_add(block[2]);
    block[0] ^= i;

    for _ in 0..block[3] {
        let (mul, _) = block[2].overflowing_mul(block[0]);
        let (sum, _) = mul.overflowing_add(block[1]);
        block[0] = sum % block[3];
    }

    block
}
